use std::sync::Arc;
use std::sync::Mutex;
use actix::dev::MessageResponse;
use actix::Message;

pub struct DownStreamPack {
    holder: Option<Arc::<std::sync::Mutex::<Self>>>,
    pub socket_handle: u16,
    pub data: bytes::BytesMut,
    pub entry: Arc::<std::sync::Mutex::<r9::ListHeadLock<Self>>>,
}

pub struct UpStreamPack {
    holder: Option<Arc::<std::sync::Mutex::<Self>>>,
    pub socket_handle: u16,
    pub data: bytes::BytesMut,
    pub entry: Arc::<std::sync::Mutex::<r9::ListHeadLock<Self>>>,
}

#[derive(Message)]
#[rtype(result = "()")]
pub struct DownStreamMessage {
    pub data: actix_web::web::Bytes,
}

impl DownStreamPack {

    pub fn new(socket_handle: u16, data: bytes::BytesMut) -> Arc<Mutex<Self>> {
        let p = Arc::new(std::sync::Mutex::new(DownStreamPack{
            holder: Default::default(),
            socket_handle: socket_handle,
            data: data,
            entry: r9::ListHeadLock::<DownStreamPack>::new_node(Default::default()),
        }));
        p.lock().unwrap().holder = Some(p.clone());
        p.lock().unwrap().entry = r9::ListHeadLock::<DownStreamPack>::new_node(Arc::downgrade(&p));
        return p;
    }

}

impl UpStreamPack {

    pub fn new(socket_handle: u16, data: bytes::BytesMut) -> Arc<Mutex<Self>> {
        let p = Arc::new(std::sync::Mutex::new(UpStreamPack{
            holder: Default::default(),
            socket_handle: socket_handle,
            data: data,
            entry: r9::ListHeadLock::<Self>::new_node(Default::default()),
        }));
        p.lock().unwrap().holder = Some(p.clone());
        p.lock().unwrap().entry = r9::ListHeadLock::<Self>::new_node(Arc::downgrade(&p));
        return p;
    }

}