use std::rc::{Rc, Weak};
use std::cell::RefCell;
use crate::cs::cs_proto as cs;
use crate::cs::cs_msgid as cs_msgid;

pub struct Handler {
    msg_id: i32,
    handler_id: i32,
    cb: Box::<dyn Fn (&f9::MsgHdr, &dyn MsgHandler)>,
}

static mut HANDLERS: Vec<Option<Handler>> = Vec::new();

pub fn get_net_msg_handler(msg_id: u16) -> &'static Option<Handler> {
    unsafe {
        if (msg_id as usize) < HANDLERS.len() {
            return &HANDLERS[msg_id as usize];
        } else {
            return &None;
        }
    }
}

pub fn reg_handler_id(msg_id: u16, handler_id: i32) {
    unsafe {
        if (msg_id as usize) < HANDLERS.len() {
            match &mut HANDLERS[msg_id as usize] {
                Some(v) => {
                    v.handler_id = handler_id;
                }
                None => {

                }
            }
        }
    }
}

pub fn dispatch_msg(msg_id: u16, hdr: &f9::MsgHdr, h: &dyn MsgHandler) {
    let mut handler: &Option<Handler> = &None;
    unsafe {
        handler = get_net_msg_handler(msg_id);
    }
    match handler {
        Some(v) => {
            (*v.cb)(hdr, h);
        }
        None => {

        }
    }
}

/*

func ParsePb(msgId uint16, data []byte) interface{} {
        handler := handlers[msgId]
        if handler == nil {
                return nil
        }
        return handler.ParseCb(data)
}*/

pub trait MsgHandler {

    fn cm_ping(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMPing>>) {
        panic!("not implement");
    }


    fn cm_kcp_handshake(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMKcpHandshake>>) {
        panic!("not implement");
    }


    fn cm_join(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMJoin>>) {
        panic!("not implement");
    }


    fn cm_reconnect(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMReconnect>>) {
        panic!("not implement");
    }


    fn cm_move(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMove>>) {
        panic!("not implement");
    }


    fn cm_immediate_msg(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMImmediateMsg>>) {
        panic!("not implement");
    }


    fn cm_team_mark_target_pos(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMTeamMarkTargetPos>>) {
        panic!("not implement");
    }


    fn cm_exec_command(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMExecCommand>>) {
        panic!("not implement");
    }


    fn cm_emote(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMEmote>>) {
        panic!("not implement");
    }


    fn cm_voice(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMVoice>>) {
        panic!("not implement");
    }


    fn cm_game_over(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMGameOver>>) {
        panic!("not implement");
    }


    fn cm_watch_war(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMWatchWar>>) {
        panic!("not implement");
    }


    fn cm_leave(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMLeave>>) {
        panic!("not implement");
    }


    fn cm_revive(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMRevive>>) {
        panic!("not implement");
    }


    fn cm_cancel_revive(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMCancelRevive>>) {
        panic!("not implement");
    }


    fn cm_match_cancel(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMatchCancel>>) {
        panic!("not implement");
    }


    fn cm_match_choose(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMatchChoose>>) {
        panic!("not implement");
    }


    fn cm_match_start_game(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMatchStartGame>>) {
        panic!("not implement");
    }


    fn cm_match_cancel_start_game(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMatchCancelStartGame>>) {
        panic!("not implement");
    }


    fn cm_match_send_msg(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMatchSendMsg>>) {
        panic!("not implement");
    }


    fn cm_match_broadcast_msg(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMMatchBroadcastMsg>>) {
        panic!("not implement");
    }


    fn cm_request_bullet_dmg(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMRequestBulletDmg>>) {
        panic!("not implement");
    }


    fn cm_request_throw_dmg(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMRequestThrowDmg>>) {
        panic!("not implement");
    }


    fn cm_stow_shield(&self, _: &f9::MsgHdr, _: Rc::<RefCell::<cs::CMStowShield>>) {
        panic!("not implement");
    }

}

pub fn registe_handlers() {
    unsafe {
        HANDLERS.reserve(2000);
        for i in 0..1999 {
            HANDLERS.push(None);
        }

        HANDLERS[cs_msgid::CMMessageId_e::_CMPing as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMPing as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMPing>>>() {
                       handler.cm_ping(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMKcpHandshake as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMKcpHandshake as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMKcpHandshake>>>() {
                       handler.cm_kcp_handshake(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMJoin as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMJoin as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMJoin>>>() {
                       handler.cm_join(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMReconnect as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMReconnect as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMReconnect>>>() {
                       handler.cm_reconnect(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMove as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMove as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMove>>>() {
                       handler.cm_move(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMImmediateMsg as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMImmediateMsg as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMImmediateMsg>>>() {
                       handler.cm_immediate_msg(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMTeamMarkTargetPos as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMTeamMarkTargetPos as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMTeamMarkTargetPos>>>() {
                       handler.cm_team_mark_target_pos(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMExecCommand as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMExecCommand as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMExecCommand>>>() {
                       handler.cm_exec_command(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMEmote as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMEmote as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMEmote>>>() {
                       handler.cm_emote(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMVoice as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMVoice as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMVoice>>>() {
                       handler.cm_voice(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMGameOver as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMGameOver as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMGameOver>>>() {
                       handler.cm_game_over(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMWatchWar as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMWatchWar as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMWatchWar>>>() {
                       handler.cm_watch_war(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMLeave as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMLeave as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMLeave>>>() {
                       handler.cm_leave(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMRevive as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMRevive as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMRevive>>>() {
                       handler.cm_revive(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMCancelRevive as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMCancelRevive as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMCancelRevive>>>() {
                       handler.cm_cancel_revive(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMatchCancel as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMatchCancel as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMatchCancel>>>() {
                       handler.cm_match_cancel(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMatchChoose as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMatchChoose as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMatchChoose>>>() {
                       handler.cm_match_choose(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMatchStartGame as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMatchStartGame as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMatchStartGame>>>() {
                       handler.cm_match_start_game(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMatchCancelStartGame as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMatchCancelStartGame as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMatchCancelStartGame>>>() {
                       handler.cm_match_cancel_start_game(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMatchSendMsg as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMatchSendMsg as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMatchSendMsg>>>() {
                       handler.cm_match_send_msg(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMMatchBroadcastMsg as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMMatchBroadcastMsg as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMMatchBroadcastMsg>>>() {
                       handler.cm_match_broadcast_msg(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMRequestBulletDmg as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMRequestBulletDmg as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMRequestBulletDmg>>>() {
                       handler.cm_request_bullet_dmg(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMRequestThrowDmg as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMRequestThrowDmg as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMRequestThrowDmg>>>() {
                       handler.cm_request_throw_dmg(hdr, msg.clone());
                   }
            }),
         });

        HANDLERS[cs_msgid::CMMessageId_e::_CMStowShield as usize] = Some(
            Handler {
                msg_id: cs_msgid::CMMessageId_e::_CMStowShield as i32,
                handler_id: 0,
                cb: Box::new(|hdr: &f9::MsgHdr, handler: &dyn MsgHandler| {
                   if let Some(msg) = hdr.msg.downcast_ref::<Rc::<RefCell::<cs::CMStowShield>>>() {
                       handler.cm_stow_shield(hdr, msg.clone());
                   }
            }),
         });

    }
}