use std::rc::{Rc, Weak};
use std::cell::RefCell;

pub struct Parameter {
    param_name: Option<String>,
    param_value: Option<String>,
}

pub struct Attr {
    attr_id: Option<i32>,
    attr_cname: Option<String>,
    attr_ename: Option<String>,
}

pub struct Map {
    map_id: Option<i32>,
    template_list: Option<String>,
    map_name: Option<String>,
    map_width: Option<f32>,
    map_height: Option<f32>,
    airdrops: Option<String>,
    terminator_airdrop: Option<i32>,
    player: Option<i32>,
    refresh_robot: Option<String>,
    map_mode: Option<i32>,
    safearea: Option<String>,
    game_start_buff_list: Option<String>,
    map_pic: Option<String>,
    first_safearea_center: Option<String>,
    init_gas_ring: Option<i32>,
    airraids: Option<String>,
    car_num_limit: Option<String>,
    scale: Option<f32>,
    map_collider: Option<String>,
    world_object_file: Option<String>,
    terrain_file: Option<String>,
}

pub struct MapArea {
    map_id: Option<i32>,
    area_type: Option<i32>,
    area_subtype: Option<i32>,
    area_center: Option<String>,
    area_width: Option<f32>,
    area_height: Option<f32>,
}

pub struct MapThing {
    thing_id: Option<i32>,
    thing_type: Option<i32>,
    time: Option<i32>,
    type: Option<i32>,
    height: Option<i32>,
    width: Option<i32>,
    hp: Option<i32>,
    damage: Option<f32>,
    damage_dia: Option<f32>,
    drop: Option<String>,
    is_door: Option<i32>,
    is_house: Option<i32>,
    is_tree: Option<i32>,
    house_id: Option<i32>,
    buff_list: Option<String>,
    explosion_effect: Option<i32>,
    explosion_interval: Option<i32>,
    explosion_dmg_delay: Option<i32>,
    explosion_times: Option<i32>,
    explosion_float: Option<i32>,
    preexplosion_summon: Option<String>,
    monster_list: Option<String>,
    special_damage_type: Option<String>,
    receive_special_damage_type: Option<String>,
    param1: Option<String>,
    param2: Option<String>,
    interaction_mode: Option<i32>,
    view_mode: Option<i32>,
    bullet_hit: Option<i32>,
    collision_hit: Option<i32>,
    explosion_hit: Option<i32>,
    sweep_tags: Option<String>,
    prebattle_hide: Option<i32>,
    life_time: Option<i32>,
    summon_born_rad: Option<i32>,
    shapes: Option<String>,
    delay_destroy: Option<i32>,
}

pub struct SafeArea {
    id: Option<i32>,
    level: Option<i32>,
    rad: Option<i32>,
    wait_time: Option<i32>,
    shrink_speed: Option<i32>,
    hurt: Option<f32>,
    type: Option<i32>,
    x1: Option<i32>,
    y1: Option<i32>,
}

pub struct SafeAreaPos {
    id: Option<i32>,
    x1: Option<i32>,
    y1: Option<i32>,
    x2: Option<i32>,
    y2: Option<i32>,
    x3: Option<i32>,
    y3: Option<i32>,
    x4: Option<i32>,
    y4: Option<i32>,
    x5: Option<i32>,
    y5: Option<i32>,
    x6: Option<i32>,
    y6: Option<i32>,
    x7: Option<i32>,
    y7: Option<i32>,
    x8: Option<i32>,
    y8: Option<i32>,
    x9: Option<i32>,
    y9: Option<i32>,
    x10: Option<i32>,
    y10: Option<i32>,
}

pub struct Item {
    id: Option<i32>,
    type: Option<i32>,
    sub_type: Option<i32>,
    quality: Option<i32>,
    use: Option<i32>,
    skinid: Option<i32>,
    isdefaultskin: Option<i32>,
    playerid: Option<i32>,
    relationship: Option<i32>,
    name: Option<String>,
}

pub struct Equip {
    id: Option<i32>,
    equip_type: Option<i32>,
    equip_subtype: Option<i32>,
    equip_lv: Option<i32>,
    fire_mode: Option<i32>,
    use_bullet: Option<i32>,
    clip_volume: Option<i32>,
    reload_time: Option<i32>,
    fire_rate: Option<i32>,
    atk: Option<String>,
    def: Option<i32>,
    max_hp: Option<String>,
    explosion_range: Option<i32>,
    bullet_speed: Option<i32>,
    range: Option<i32>,
    use_time: Option<i32>,
    heal: Option<String>,
    time: Option<i32>,
    volume: Option<String>,
    bullet_rad: Option<i32>,
    group_num: Option<i32>,
    is_luck: Option<i32>,
    bullet_born_offset: Option<String>,
    bullet_angle: Option<f32>,
    name: Option<String>,
    rad: Option<f32>,
    rad2: Option<f32>,
    buffid: Option<i32>,
    drop_id: Option<i32>,
    explosion_effect: Option<i32>,
    param1: Option<String>,
    param2: Option<String>,
    reloadtype: Option<i32>,
    recoil_force: Option<f32>,
    missiles_time: Option<i32>,
    heroid: Option<i32>,
    launch_dev: Option<String>,
    power_time: Option<String>,
    power_gun: Option<String>,
    power_buff: Option<String>,
    through_teammate: Option<i32>,
    text_icon: Option<String>,
    special_damage_type: Option<String>,
    max_oil: Option<f32>,
    average_oil: Option<f32>,
    atk_mech: Option<f32>,
    use_scene: Option<i32>,
    ispenetrate: Option<i32>,
    is_penetrate_thing: Option<i32>,
    reload_delay_time: Option<i32>,
    cast_time: Option<i32>,
    aiming_cast_time: Option<i32>,
    auto_switch_weapon_time: Option<i32>,
    quality: Option<i32>,
    explosion_damage_delay: Option<i32>,
    bullet_consume_type: Option<i32>,
    inventory_slot: Option<String>,
    _inventory_slot: Option<i32>,
    critical: Option<f32>,
    cri_damage: Option<f32>,
    shootfire: Option<i32>,
    hit_buff: Option<String>,
    auto_trace: Option<i32>,
    trace_range: Option<i32>,
    double_gun: Option<i32>,
}

pub struct Hero {
    id: Option<i32>,
    radius: Option<f32>,
    move_speed: Option<f32>,
    jump_speed: Option<f32>,
    fall_speed: Option<f32>,
    aim_speed: Option<f32>,
    shoot_speed: Option<f32>,
    reload_speed: Option<f32>,
    medicine_speed: Option<f32>,
    swim_speed: Option<f32>,
    volume: Option<String>,
    level: Option<i32>,
    race: Option<i32>,
    name: Option<String>,
    hit_offset_x: Option<f32>,
    hit_offset_y: Option<f32>,
    hit_radius: Option<f32>,
    ai_script: Option<String>,
    init_buffs: Option<String>,
    default_weapon: Option<i32>,
    dead_drop: Option<String>,
    delay_delete: Option<i32>,
    ai: Option<i32>,
    delay_remove: Option<i32>,
    skinlist: Option<i32>,
    pre_appear_effect: Option<String>,
    pve_score: Option<String>,
    hp: Option<i32>,
    damage: Option<i32>,
    defence: Option<i32>,
    crit_atk: Option<f32>,
    crit_atk_ratio: Option<f32>,
    miss: Option<f32>,
    miss_damage_ruduce: Option<f32>,
    skill1list: Option<i32>,
    skill2list: Option<i32>,
    skill3list: Option<i32>,
    hp_ratio: Option<f32>,
    damage_ratio: Option<f32>,
    defence_ratio: Option<f32>,
    bt: Option<String>,
}

pub struct Robot {
    id: Option<i32>,
    name: Option<String>,
    hero_id: Option<i32>,
    skin_id: Option<String>,
    weapon_id: Option<i32>,
    weapon_lv: Option<i32>,
    sex: Option<i32>,
    dmg_ratio: Option<f32>,
    bullet_offset: Option<String>,
}

pub struct Skill {
    skill_id: Option<i32>,
    skill_type: Option<i32>,
    value_up: Option<String>,
    skill_cd: Option<i32>,
    skill_target: Option<i32>,
    buff_list: Option<String>,
    skill_distance: Option<f32>,
    cold_time_up: Option<i32>,
    max_times: Option<i32>,
    phase1_time_offset: Option<i32>,
    phase1_func: Option<i32>,
    phase1_param1: Option<String>,
    phase1_param2: Option<String>,
    phase1_param3: Option<String>,
    phase2_time_offset: Option<i32>,
    phase2_func: Option<i32>,
    phase2_param1: Option<String>,
    phase2_param2: Option<String>,
    phase2_param3: Option<String>,
    phase3_time_offset: Option<i32>,
    phase3_func: Option<i32>,
    phase3_param1: Option<String>,
    phase3_param2: Option<String>,
    phase3_param3: Option<String>,
    cast_time: Option<i32>,
    up_exp: Option<i32>,
    nextlv_skill: Option<i32>,
    attack_dir_lock_time: Option<i32>,
}

pub struct SkillNumber {
    skill_id: Option<i32>,
    skill_type: Option<i32>,
    number: Option<f32>,
    damage: Option<f32>,
    damage_addition: Option<f32>,
    damage_change: Option<f32>,
    shield: Option<f32>,
    shield_addition: Option<f32>,
    resume: Option<f32>,
    resume_addition: Option<f32>,
    speed: Option<String>,
    range: Option<String>,
    range2: Option<String>,
    range3: Option<String>,
    time: Option<String>,
    cd: Option<String>,
    buff_id: Option<i32>,
    buff_time: Option<f32>,
    probability: Option<String>,
    explosion_effect: Option<i32>,
    effect_list: Option<String>,
}

pub struct NpcStandard {
    id: Option<i32>,
    quality: Option<i32>,
    hp: Option<i32>,
    damage: Option<i32>,
    defence: Option<i32>,
}

pub struct Buff {
    buff_id: Option<i32>,
    name: Option<String>,
    buff_target: Option<i32>,
    buff_effect: Option<i32>,
    trigger_type: Option<i32>,
    buff_param1: Option<String>,
    buff_param2: Option<String>,
    buff_param3: Option<String>,
    buff_param4: Option<String>,
    buff_param5: Option<String>,
    buff_param6: Option<String>,
    duration_time: Option<String>,
    buff_valueup: Option<f32>,
    immune_buffeffect_list: Option<String>,
    post_remove_action: Option<String>,
    only_server: Option<i32>,
    only_self: Option<i32>,
    depend_effect: Option<i32>,
    child_buff: Option<String>,
    coexist_num: Option<i32>,
    dead_valid: Option<i32>,
    buff_interval: Option<String>,
    tag: Option<String>,
    post_battle_valid: Option<i32>,
    only_spec_race: Option<String>,
    exclude_spec_race: Option<String>,
    dead_remove: Option<i32>,
    no_immune: Option<i32>,
    lock_move: Option<i32>,
    lock_dir: Option<i32>,
    lock_move_dir: Option<i32>,
    disable_shot: Option<i32>,
    disable_useskill: Option<i32>,
    effect_list: Option<String>,
    res_scale: Option<String>,
    tenacity: Option<i32>,
}

pub struct Drop {
    drop_id: Option<i32>,
    item_id: Option<String>,
    num: Option<String>,
    weight: Option<String>,
    type: Option<i32>,
}

pub struct AirDrop {
    id: Option<i32>,
    time: Option<i32>,
    appear_time: Option<i32>,
    drop_id: Option<String>,
}

pub struct AirRaid {
    id: Option<i32>,
    time: Option<i32>,
    appear_time: Option<i32>,
    bomb_id: Option<String>,
    raid_wave: Option<String>,
    rad: Option<f32>,
}

pub struct AirLine {
    id: Option<i32>,
    start_point: Option<String>,
    end_point: Option<String>,
    plane_speed: Option<f32>,
    weight: Option<i32>,
    map_id: Option<i32>,
}

pub struct RankReward {
    rank: Option<i32>,
    parameter: Option<f32>,
    drop: Option<i32>,
}

pub struct RankPoint {
    rank: Option<i32>,
    parameter: Option<i32>,
    parameter2: Option<i32>,
    parameter3: Option<i32>,
}

pub struct KillReward {
    kill_num: Option<i32>,
    parameter: Option<f32>,
}

pub struct KillPoint {
    kill_num: Option<i32>,
    parameter: Option<i32>,
    parameter2: Option<i32>,
}

pub struct AI {
    id: Option<i32>,
    pursuit_radius: Option<i32>,
    attack_interval: Option<i32>,
    attack_times: Option<i32>,
    attack_type: Option<i32>,
    shot_offset_angle: Option<i32>,
    random_move_idle_time: Option<String>,
    random_move_time: Option<String>,
    attack_range: Option<i32>,
    attack_rate: Option<f32>,
    ai_mode: Option<i32>,
    ai_kind: Option<i32>,
    param1: Option<String>,
    param2: Option<String>,
    param3: Option<String>,
    param4: Option<String>,
    param5: Option<String>,
    peace_time: Option<i32>,
}

pub struct Text {
    textid: Option<String>,
    text: Option<String>,
}

pub struct GunTalentGrow {
    id: Option<i32>,
    talent_id: Option<i32>,
    talent_lv: Option<i32>,
    addtype: Option<i32>,
    addattr: Option<String>,
}

pub struct HeroQuality {
    id: Option<i32>,
    quality: Option<i32>,
    gold_limit: Option<i32>,
    lucky: Option<i32>,
}

pub struct GunQuality {
    id: Option<i32>,
    quality: Option<i32>,
    gold_limit: Option<i32>,
    lucky: Option<i32>,
}

pub struct FormulaPvp {
    top: Option<f32>,
    ranked_topx: Option<f32>,
    kills_topx: Option<f32>,
    hero_topx: Option<f32>,
    weapon_topx: Option<f32>,
    survival_topx: Option<f32>,
}

pub struct PveGemini {
    gemini_id: Option<i32>,
    gemini_lv: Option<i32>,
    gemini_limit: Option<i32>,
    multiplayer_enemy_hp_mul: Option<String>,
    born_point: Option<String>,
}

pub struct PveGeminiContent {
    id: Option<i32>,
    mode_id: Option<i32>,
    round: Option<i32>,
    spawn_point: Option<String>,
    enemy_id: Option<String>,
}

pub struct PveGeminiMode {
    id: Option<i32>,
    map_id: Option<i32>,
    mode_time: Option<String>,
    area: Option<String>,
    score_reward: Option<String>,
    round_score: Option<String>,
    next_door: Option<String>,
    wave_prepare_time: Option<i32>,
}

pub struct RankRoom {
    id: Option<i32>,
    elo_min: Option<i32>,
    elo_max: Option<i32>,
    elo_expansion1: Option<i32>,
    elo_expansion2: Option<i32>,
    expand_time1: Option<i32>,
    expand_time2: Option<i32>,
    player_num_standard: Option<i32>,
    final_time: Option<i32>,
    final_player_num: Option<i32>,
}

pub struct Grasp {
    grasp_id: Option<i32>,
    hero_id: Option<i32>,
    hero_lv: Option<i32>,
    graspbuff_id1: Option<i32>,
    graspbuff_id1_floor2: Option<i32>,
    graspbuff_id2: Option<i32>,
    weapon_id: Option<i32>,
    add_buff_list: Option<String>,
    remove_buff_list: Option<String>,
}

pub struct GraspBuff {
    graspbuff_id: Option<i32>,
    graspbuff_floor: Option<i32>,
    graspbuff_trigger: Option<String>,
    graspbuff_target: Option<i32>,
    graspbuff_time: Option<String>,
    attr_id: Option<i32>,
    attr_add_pattern: Option<i32>,
    attr_add_pattern2: Option<i32>,
    attr_num: Option<String>,
    effect_list: Option<String>,
}

pub struct GuideStep {
    id: Option<i32>,
    target: Option<i32>,
    param1: Option<String>,
    param2: Option<String>,
    param3: Option<String>,
    param4: Option<String>,
}

pub struct WorldObject {
    object_id: Option<i32>,
    object_type: Option<i32>,
    x: Option<f32>,
    y: Option<f32>,
    z: Option<f32>,
}

