use std::collections::HashMap;
use std::rc::Rc;
use tokio::runtime::Runtime;
use tokio::net::TcpStream;
use std::cell::RefCell;
use std::sync::{Arc, Mutex};
use crate::upstream::UpStream;
use crate::app::UserApp;
use crate::common::{DownStreamPack, UpStreamPack};

pub struct UpStreamMgr {
    curr_id: i16,
    key_hash: RefCell::<HashMap<String, Rc::<UpStream>>>,
    id_hash: RefCell::<HashMap<i32, Rc::<UpStream>>>,
    tokio_rt: Runtime,
}
unsafe impl Send for UpStreamMgr{}
unsafe impl Sync for UpStreamMgr{}
impl UpStreamMgr {

    pub fn instance() -> &'static Self {
        return &UserApp::instance()._upstream_mgr;
    }

    pub fn new() -> Self {
        let p = Self{
            curr_id: 0,
            key_hash: Default::default(),
            id_hash: Default::default(),
            tokio_rt: tokio::runtime::Builder::new_multi_thread()
                .enable_all()
                .build()
                .unwrap(),
        };
        return p;
    }

    pub fn init(&self) {
        let upstream = Rc::new(UpStream::new(1, "192.168.100.39".to_string(), 7616));

        self.key_hash.borrow_mut().insert("".to_string(), upstream.clone());
        self.id_hash.borrow_mut().insert(upstream.instance_id, upstream.clone());
        upstream.init(&self.tokio_rt);
    }

    pub fn uninit(&self) {

    }

    pub fn add_down_stream_pack(&self, data: std::sync::Weak::<std::sync::Mutex::<DownStreamPack>>) {
        println!("add_down_stream_pack2 len:{0}", data.upgrade().unwrap().lock().unwrap().data.len());
        let socket_handle = data.upgrade().unwrap().lock().unwrap().socket_handle;
        for (key, value) in self.id_hash.borrow().iter() {
            value.down_pack_queue.lock().unwrap().push(&data.upgrade().unwrap().lock().unwrap().entry);
        }
    }

    pub fn update(&self) {
        for (key, value) in self.id_hash.borrow().iter() {
            value.update();
        }
    }

}
